from datetime import datetime, timedelta, UTC
from typing import Any

import jwt

DEFAULT_EXPIRATION = 60 * 60  # 60 minutes
ALGORITHM = "HS256"

DecodeError = jwt.DecodeError
ExpiredSignatureError = jwt.ExpiredSignatureError


def create_expiration_dt(seconds: int) -> datetime:
    return datetime.now(UTC) + timedelta(seconds=seconds)


def encode(
    *,
    data: dict[str, Any],
    secret: str,
    expires_at: datetime | None = None,
    expires_in: int | None = DEFAULT_EXPIRATION
) -> str:

    to_encode = data.copy()
    if not expires_at:
        expires_in = expires_in or DEFAULT_EXPIRATION
        expires_at = create_expiration_dt(seconds=expires_in)

    to_encode["exp"] = expires_at
    return jwt.encode(to_encode, secret, algorithm=ALGORITHM)


def decode_unsafe(*, token: str, secret: str) -> dict[str, Any]:
    return jwt.decode(token, secret, algorithms=[ALGORITHM])


def decode(
    *,
    token: str,
    secret: str,
) -> dict[str, Any]:
    res = decode_unsafe(token=token, secret=secret)

    if res.get("type", "") != type:
        raise Exception(
            "JWT of unexpected type, expected '%s' got '%s'", type, res.get("type", "")
        )

    return res



import datetime
from ulid import ULID


def id_generator() -> ULID:
    """ULID ID generator."""
    return ULID()


def id_generator_with_timestamp(timestamp: int | float) -> ULID:
    return ULID.from_timestamp(timestamp)


def id_generator_with_datetime(dt: datetime.datetime) -> ULID:
    return ULID.from_datetime(dt)


def id_generator_string() -> str:
    return str(id_generator())


def id_generator_int() -> int:
    return int(id_generator())


def id_generator_hex() -> str:
    return id_generator().hex


def id_generator_timestamp() -> float:
    return id_generator().timestamp


def id_generator_datetime() -> datetime:
    return id_generator().datetime


def id_generator_uuid() -> str:
    return str(id_generator().to_uuid())



import threading
from functools import wraps


def singleton(cls):
    """
    单例装饰器
    """
    instances = {}
    lock = threading.Lock()  # 锁对象，确保线程安全

    def get_instance(*args, **kwargs):
        if cls not in instances:  # 第一次检查（无锁）
            with lock:
                if cls not in instances:  # 第二次检查（有锁）
                    instances[cls] = cls(*args, **kwargs)
        return instances[cls]

    return get_instance


def sync_to_async(func):
    """
    将同步函数转换为异步函数
    """

    @wraps(func)
    async def async_wrapper(*args, **kwargs):
        return func(*args, **kwargs)

    return async_wrapper


from typing import Any, Set


def get_pydantic_field_names(pydantic_cls: Any) -> Set[str]:
    """Get field names, including aliases, for a pydantic class.

       Args:
           pydantic_cls: Pydantic class.

       Returns:
           Set[str]: Field names.
    """
    all_required_field_names = set()
    for name, field in pydantic_cls.model_fields.items():
        all_required_field_names.add(name)
        if field.alias:
            all_required_field_names.add(field.alias)

    return all_required_field_names



import os
from functools import lru_cache

from dotenv import load_dotenv
from pydantic_settings import BaseSettings, SettingsConfigDict
from typing import Optional, Literal


def get_env_file(env: Optional[str] = None) -> str:
    """
    获取环境的 .env 文件路径。
    如果未指定环境参数，则尝试从系统环境变量 APP_ENV 中获取环境名。
    如果未找到 APP_ENV，则返回默认的 `.env` 文件。
    """

    env = env or os.getenv('APP_ENV')

    if env is None:
        # 如果没有传递任何环境，并且系统环境变量 `APP_ENV` 也未设置，则使用默认的 `.env` 文件
        return ".env"

    # 如果明确指定了环境，加载对应的 `.env.{env}` 文件
    return f".env.{env}"


# 在程序启动时只加载一次 .env 文件
@lru_cache()
def load_env_file():
    load_dotenv(get_env_file())


class VovoBaseSettings(BaseSettings):
    # 应用的配置项

    PROJECT_NAME: str = 'Vovo API'
    DOMAIN: str = 'http://localhost:8080'
    ENVIRONMENT: Literal["local", "staging", "production"] = "local"

    VERSION: str = '1.0.0'
    DEBUG: bool = False

    SENTRY_URI: str | None = None
    CORS_ORIGINS: str = "*"

    CELERY_NAME: str = 'vovo celery'
    CELERY_BROKER: str = 'redis://localhost:6379/0'
    CELERY_RESULT_BACKEND: str = 'redis://localhost:6379/0'
    CELERY_TASK_SERIALIZER: str = 'json'

    # 使用 ConfigDict 替代 class-based Config
    model_config = SettingsConfigDict(
        env_file=get_env_file(),
        env_file_encoding='utf-8',
        extra="allow"  # 允许 .env 文件中存在未定义的字段
    )

    def __init__(self, env: Optional[str] = None, **kwargs):
        # 确保每次实例化时，.env 文件已经加载
        load_env_file()
        super().__init__(**kwargs)


@lru_cache()
def get_global_settings() -> VovoBaseSettings:
    """
       This function returns a cached instance of the VovoBaseSettings object.

       Caching is used to prevent re-reading the environment every time the API settings are used in an endpoint.

       If you want to change an environment variable and reset the cache (e.g., during testing), this can be done
       using the `lru_cache` instance method `get_api_settings.cache_clear()`.
    """

    return VovoBaseSettings()


global_settings = get_global_settings()


from functools import wraps

from sqlmodel.ext.asyncio.session import AsyncSession


def transactional(f: Callable):
    """定义事务装饰器"""

    @wraps(f)
    def wrapper(*args, db: AsyncSession, **kwargs):
        try:
            # 调用被装饰的函数
            result = f(*args, db=db, **kwargs)
            # 提交事务
            db.commit()
            return result
        except Exception as e:
            # 回滚事务
            db.rollback()
            raise HTTPException(status_code=500, detail="An error occurred during the transaction") from e

    return wrapper



import structlog
from starlette.middleware.cors import CORSMiddleware
from starlette.types import ASGIApp, Scope, Receive, Send

from vovo.custom_logger import generate_correlation_id
from vovo.settings import global_settings


def add_cors_middleware(app):
    """cors中间件"""
    origins = [str(origin).strip("/") for origin in global_settings.CORS_ORIGINS]
    app.add_middleware(
        CORSMiddleware,
        allow_origins=origins,  # 允许的来源
        allow_credentials=True,  # 允许发送 cookies
        allow_methods=["*"],  # 允许的 HTTP 方法，例如 GET, POST 等
        allow_headers=["*"],  # 允许的 HTTP 头
    )


class LogCorrelationIdMiddleware:
    def __init__(self, app: ASGIApp) -> None:
        self.app = app

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        if scope["type"] != "http":
            return await self.app(scope, receive, send)

        structlog.contextvars.bind_contextvars(
            correlation_id=generate_correlation_id(),
            method=scope["method"],
            path=scope["path"],
        )

        await self.app(scope, receive, send)

        structlog.contextvars.unbind_contextvars("correlation_id", "method", "path")



from abc import ABC, abstractmethod
from typing import Dict, Any


class AbstractHttpClient(ABC):
    """
    HTTP 客户端的抽象基类，异步请求的方法。
    """

    @abstractmethod
    async def request_async(self, method: str, url: str, params: Dict[str, Any] = None,
                            data: Dict[str, Any] = None, headers: Dict[str, str] = None) -> Any:
        """
        异步 HTTP 请求

        :param method: HTTP 方法（GET, POST, PUT, DELETE 等）
        :param url: 请求的相对或完整 URL
        :param params: URL 查询参数，可选
        :param data: 请求体数据，可选
        :param headers: 自定义请求头，可选
        :return: 返回响应的数据
        """
        pass




import os

import sentry_sdk


def configure_sentry(sentry_uri=None, traces_sample_rate=1.0, profiles_sample_rate=1.0):
    """
    初始化 Sentry 监控配置

    :param sentry_uri: Sentry DSN URI。如果未提供，则从环境变量 'SENTRY_URI' 中获取。
    :param traces_sample_rate: 性能监控的采样率。默认设置为 1.0，表示捕获 100% 的事务。
    :param profiles_sample_rate: 采样事务的性能分析采样率。默认设置为 1.0，表示对采样的 100% 事务进行分析。
    """

    # 如果未传递 sentry_uri 参数，则从环境变量中获取 SENTRY_URI
    if sentry_uri is None:
        sentry_uri = os.getenv('SENTRY_URI')

    # 如果找到 SENTRY_URI，初始化 Sentry 监控
    if sentry_uri:
        sentry_sdk.init(
            dsn=sentry_uri,
            # 设置 traces_sample_rate 为 1.0，以捕获 100% 的事务用于性能监控
            traces_sample_rate=traces_sample_rate,
            # 我们建议在生产环境中调整此值以减少开销
            profiles_sample_rate=profiles_sample_rate,
        )
    else:
        raise ValueError("未找到 SENTRY_URI")


from vovo.persistence.models import Page, BaseMongoModel
from vovo.persistence.repositories import GenericRepository, GenericSqlRepository, MongoDBRepository

__all__ = [
    'Page',
    'BaseMongoModel',
    'GenericRepository',
    'GenericSqlRepository',
    'MongoDBRepository'
]


import re
from typing import Any, List


pattern_uppercase = re.compile(r"([A-Z]+)([A-Z][a-z])")
pattern_camel_case = re.compile(r"([a-z\d])([A-Z])")


def str_to_snake(string: str) -> str:
    """
    Examples:
    -------
    >>> str_to_snake("myCamelName") == my_camel_name"
    assert str_to_snake("snake-case-example") == "snake_case_example"
    assert str_to_snake("AnotherExample123") == "another_example123"
    assert str_to_snake("PascalCase") == "pascal_case"
    """
    string = pattern_uppercase.sub(r"\1_\2", string)
    string = pattern_camel_case.sub(r"\1_\2", string)
    string = string.replace("-", "_")
    return string.lower()


def stringify_value(val: Any) -> str:
    """Stringify a value.

    Args:
        val: The value to stringify.

    Returns:
        str: The stringifies value.
    """
    if isinstance(val, str):
        return val
    elif isinstance(val, dict):
        return "\n" + stringify_dict(val)
    elif isinstance(val, list):
        return "\n".join(stringify_value(v) for v in val)
    else:
        return str(val)


def stringify_dict(data: dict) -> str:
    """Stringify a dictionary.

    Args:
        data: The dictionary to stringify.

    Returns:
        str: The stringifies dictionary.
    """
    text = ""
    for key, value in data.items():
        text += key + ": " + stringify_value(value) + "\n"
    return text


def join_with_separator(items: List[Any], separator: str = ", ") -> str:
    """Convert a list to a string separated by a custom separator.

    Args:
        items: The list to convert.
        separator: The string to use as a separator, default is ', '.

    Returns:
        str: The separated string.
    """
    return separator.join(str(item) for item in items)



import threading
import time
from functools import lru_cache
from typing import Type, List

from sqlalchemy import inspect

from vovo.core.decorators import singleton


@lru_cache(maxsize=256)  # 设置缓存的最大大小（256个模型）
def get_primary_keys(model: Type) -> List[str]:

    model_inspector = inspect(model)  # 直接使用 SQLAlchemy 的 inspect
    primary_keys = [key.name for key in model_inspector.mapper.primary_key]  # 获取所有主键的列名
    return primary_keys


@singleton
class IDGen:
    def __init__(self, datacenter_id=0, machine_id=0, epoch=1704067200000):
        """
        IDGen 构造函数，支持 datacenter_id 和 machine_id 缺省
        :param datacenter_id: 数据中心ID（默认值为0）
        :param machine_id: 机器ID（默认值为0）
        :param epoch: 自定义纪元时间（默认是 Twitter 起始时间）
        """
        # 校验参数合法性
        if datacenter_id > 31 or datacenter_id < 0:
            raise ValueError("datacenter_id must be between 0 and 31")
        if machine_id > 31 or machine_id < 0:
            raise ValueError("machine_id must be between 0 and 31")

        self.datacenter_id = datacenter_id
        self.machine_id = machine_id
        self.epoch = epoch  # 起始时间戳（自定义纪元）
        self.sequence = 0  # 序列号
        self.last_timestamp = -1  # 记录上次生成ID时的时间戳

        # 位数定义
        self.datacenter_id_bits = 5
        self.machine_id_bits = 5
        self.sequence_bits = 12

        # 最大值定义
        self.max_sequence = -1 ^ (-1 << self.sequence_bits)  # 4095

        # 位移定义
        self.machine_id_shift = self.sequence_bits
        self.datacenter_id_shift = self.sequence_bits + self.machine_id_bits
        self.timestamp_shift = self.sequence_bits + self.machine_id_bits + self.datacenter_id_bits

        # 锁，用于确保线程安全
        self.lock = threading.Lock()

    @staticmethod
    def _current_timestamp():
        """获取当前时间戳，单位毫秒"""
        return int(time.time() * 1000)

    def _wait_for_next_millisecond(self, last_timestamp):
        """等待直到下一毫秒"""
        timestamp = self._current_timestamp()
        while timestamp <= last_timestamp:
            timestamp = self._current_timestamp()
        return timestamp

    def generate_id(self):
        """生成唯一ID"""
        with self.lock:
            timestamp = self._current_timestamp()

            # 如果当前时间小于上一次生成ID的时间，系统时钟回拨了
            if timestamp < self.last_timestamp:
                raise Exception("Clock moved backwards. Refusing to generate id")

            # 同一毫秒内，递增序列号
            if timestamp == self.last_timestamp:
                self.sequence = (self.sequence + 1) & self.max_sequence
                # 如果序列号溢出，等待下一毫秒
                if self.sequence == 0:
                    timestamp = self._wait_for_next_millisecond(self.last_timestamp)
            else:
                # 如果时间戳变化了，序列号重置为0
                self.sequence = 0

            # 更新最后生成ID的时间戳
            self.last_timestamp = timestamp

            # 生成ID
            unique_id = (
                    (timestamp - self.epoch) << self.timestamp_shift
                    | (self.datacenter_id << self.datacenter_id_shift)
                    | (self.machine_id << self.machine_id_shift)
                    | self.sequence
            )

            return unique_id



from typing import Union, Any


def get_type(value: Any) -> Union[str, type]:
    """
    从字典中检索类型值

    Returns:
        The type value.
    """
    # get "type" or "annotation" from the value
    _type = value.get("type") or value.get("annotation")

    return _type if isinstance(_type, str) else _type.__name__



import asyncio
import threading


def run_until_complete(coro):
    try:
        loop = asyncio.get_event_loop()
        if loop.is_running():
            # 在一个新的线程中运行一个单独的事件循环来执行协程
            return run_in_thread(coro)
        else:
            return loop.run_until_complete(coro)
    except RuntimeError:
        # 如果没有事件循环，则创建一个新的事件循环并运行协程
        return asyncio.run(coro)


def run_in_thread(coro):
    result = None
    exception = None

    def target():
        nonlocal result, exception
        try:
            result = asyncio.run(coro)
        except Exception as e:
            exception = e

    thread = threading.Thread(target=target)
    thread.start()
    thread.join()

    if exception:
        raise exception
    return result



from abc import ABC, abstractmethod
from typing import Callable


class BaseKeyMaker(ABC):
    @abstractmethod
    async def make(self, *, function: Callable, prefix: str) -> str:
        """Base key maker"""



import pickle
from typing import Any
import json
from redis import asyncio as redis

from vovo.cache.base.backend import BaseBackend


class RedisBackend(BaseBackend):
    """Redis backend for caching."""

    def __init__(self, redis_url: str):
        self.redis_client = redis.from_url(url=redis_url, decode_responses=True)

    async def get(self, *, key: str) -> Any:
        result = await self.redis_client.get(key)
        if not result:
            return

        return json.loads(result)

    async def set(self, *, response: Any, key: str, ttl: int = 60) -> None:
        if isinstance(response, dict):
            response = json.dumps(response)
        else:
            response = json.dumps(response)

        await self.redis_client.set(name=key, value=response, ex=ttl)

    async def delete_startswith(self, *, value: str) -> None:
        async for key in self.redis_client.scan_iter(f"{value}*"):
            await self.redis_client.delete(key)



from datetime import datetime, timezone
from typing import TypeVar, Any, List

from pydantic import BaseModel
from pymongo import ASCENDING, DESCENDING

T = TypeVar("T", bound=BaseModel)


def set_timestamps(record: T, is_new: bool = False):
    """Sets the created_at, updated_at fields for a record.

    Args:
        record (T): The record to update timestamps for.
        is_new (bool, optional): Whether the record is new. Defaults to False.
    """

    # Check if the record has a created_at attribute and set it if needed
    current_time = datetime.now(timezone.utc)

    if is_new and hasattr(record, "created_at"):
        if getattr(record, "created_at") is None:
            setattr(record, "created_at", current_time)

    # Always update the updated_at field, either on creation, update, or delete
    if hasattr(record, "updated_at"):
        setattr(record, "updated_at", current_time)


# Define operator_map outside the function for reuse and better performance
OPERATOR_MAP = {
    ">=": lambda f, v: f >= v,
    "<=": lambda f, v: f <= v,
    ">": lambda f, v: f > v,
    "<": lambda f, v: f < v,
    "!=": lambda f, v: f != v,
    "in": lambda f, values: f.in_(values),
    "not in": lambda f, values: f.notin_(values),
    "like": lambda f, v: f.like(v),
    "not set": lambda f, v: None if v is None else f == v
}


def apply_operator(field, operator: str, value: Any):
    """Applies an operator to a field based on the provided operator string."""

    operator = operator.lower()

    # Handle the `in` and `not in` operators separately to ensure we pass a list or set
    if operator in ["in", "not in"]:
        if not isinstance(value, (list, set, tuple)):
            raise ValueError(f"`{operator}` operator expects a list, set, or tuple, got {type(value).__name__}")

    apply_op = OPERATOR_MAP.get(operator)

    if apply_op is None:
        raise ValueError(f"Unsupported operator: {operator}")

    # Apply the operator and return the condition
    return apply_op(field, value)


def mongo_build_sort(order_by: str | List[str]) -> List[tuple[str, int]]:
    """Build MongoDB sort parameters."""
    if isinstance(order_by, str):
        order_by = [order_by]

    sort_fields = []
    for field in order_by:
        if field.startswith("-"):
            sort_fields.append((field[1:], DESCENDING))
        else:
            sort_fields.append((field, ASCENDING))

    return sort_fields



from abc import ABC, abstractmethod
from typing import Generic, TypeVar, Any, Type, List

from bson import ObjectId
from motor.motor_asyncio import AsyncIOMotorCollection, AsyncIOMotorDatabase

from pydantic import BaseModel
from sqlalchemy import func
from sqlmodel import select, desc, asc, and_, or_
from sqlmodel.ext.asyncio.session import AsyncSession

from vovo.persistence.models import Page, BaseMongoModel
from vovo.persistence.utils import set_timestamps, apply_operator, mongo_build_sort
from vovo.utils.orm import get_primary_keys

T = TypeVar("T", bound=BaseModel)


class GenericRepository(Generic[T], ABC):
    """Generic base repository."""

    @abstractmethod
    async def get(self, *args: Any, **kwargs: Any) -> T | None:
        """Get a single record by either positional or keyword arguments (or both).

        Args:
            *args (Any): Positional arguments representing values for filtering.
                         The order of arguments should match the expected field order.
            **kwargs (Any): Keyword arguments representing field names and their corresponding values
                            for filtering (e.g., id=1, name="John").

        Returns:
            Optional[T]: Record or None if not found.
        """
        raise NotImplementedError()

    @abstractmethod
    async def get_list(self, limit: int = 0, order_by: str | List[str] | None = None, **filters) -> list[T]:
        """Gets a list of records

        Args:
            limit (int): The maximum number of records to return. Default is 0,
                     where 0 means no limit (all records will be returned).
            order_by: 排序规则
            **filters: Filter conditions, several criteria are linked with a logical 'and'.

         Raises:
            ValueError: Invalid filter condition.

        Returns:
            List[T]: List of records.
        """
        raise NotImplementedError()

    @abstractmethod
    async def get_page_list(self, page_number: int, page_size: int, order_by: str | List[str] | None = None,
                            **filters: Any) -> Page[T]:
        """Get a paginated list of records."""
        raise NotImplementedError()

    @abstractmethod
    async def get_count(self, **filters: Any) -> int:
        """Get the count query with filters"""
        raise NotImplementedError()

    @abstractmethod
    async def add(self, record: T) -> T:
        """Creates a new record.

        Args:
            record (T): The record to be created.

        Returns:
            T: The created record.
        """
        raise NotImplementedError()

    @abstractmethod
    async def update(self, record: T) -> T:
        """Updates an existing record.

        Args:
            record (T): The record to be updated incl. record id.

        Returns:
            T: The updated record.
        """
        raise NotImplementedError()

    @abstractmethod
    async def delete(self, *args: Any, **kwargs: Any) -> None:
        """Deletes a record by either positional or keyword arguments.

        Args:
            *args (Any): Positional arguments representing values for filtering.
            **kwargs (Any): Keyword arguments representing field names and their corresponding values for filtering.
        """
        raise NotImplementedError()


class GenericSqlRepository(GenericRepository[T], ABC):

    def __init__(self, session: AsyncSession, model: Type[T]) -> None:
        """Creates a new repository instance.

        Args:
            session (AsyncSession): SQLModel async session.
            model (Type[T]): SQLModel class type.
        """
        self.session = session
        self.model = model

    async def get(self, *args: Any, **kwargs: Any) -> T | None:

        query = self._build_query(*args, **kwargs).limit(1)
        res = await self.session.exec(query)
        return res.first()

    async def get_list(self, limit: int = 0, order_by: str | List[str] | None = None, filters: dict = None,
                       **kwargs: Any) -> list[T]:
        """Gets a list of records from the database."""
        query = select(self.model)

        # Apply filters and ordering
        query = self._apply_filters_and_ordering(query, filters=filters, order_by=order_by, **kwargs)

        # Apply limit
        if limit > 0:
            query = query.limit(limit)

        res = await self.session.exec(query)

        return list(res.all())

    async def get_page_list(self, page_number: int = 1, page_size: int = 10, order_by: str | List[str] | None = None,
                            filters: dict = None, **kwargs: Any) -> Page[T]:
        """
                   获取分页列表的通用方法。

                   该方法根据分页参数、排序规则、过滤条件以及额外的简单过滤条件从数据库中获取分页列表。

                   Args:
                       page_number (int, optional):
                           当前页码，默认为 1。
                           表示要查询的页数，第一页为 1。

                       page_size (int, optional):
                           每页记录数，默认为 10。
                           表示每一页包含多少条记录。

                       order_by (str | List[str] | None, optional):
                           排序字段，可以为字符串或字符串列表，默认为 None。
                           如果字段名前有 `-` 表示降序排序，否则为升序。例如：`-created_at` 表示按 `created_at` 降序排列。

                       filters (dict, optional):
                           复杂过滤条件，支持 `and` 或 `or` 逻辑嵌套，默认为 None。
                           示例：
                           ```python
                           filters={
                               'or': [
                                   {"created_at": {">": start_of_day}},  # created_at 大于 start_of_day
                                   {"created_at": {"<": end_of_day}}     # created_at 小于 end_of_day
                               ]
                           }
                           ```

                       **kwargs (Any):
                           额外的简单过滤条件，通过 `AND` 逻辑组合应用。
                           每个条件的形式为 `{字段名: {操作符: 值}}`。
                           示例： `created_at={">": day}` 表示 `created_at > day`。
                           如果条件是 `None`，则自动转换为 `IS NULL` 过滤条件。

                   Returns:
                       Page[T]:
                           返回一个 `Page` 对象，其中包含分页结果、当前页码、每页记录数以及总记录数。
                           Page 对象的字段包括：
                           - `elements`: 当前页的数据列表
                           - `page_number`: 当前页码
                           - `page_size`: 每页记录数
                           - `total_records`: 总记录数

                   Raises:
                       ValueError:
                           如果过滤条件无效或者操作符不支持，则抛出 `ValueError`。

                   示例:
                       ```python
                       results = await self.get_page_list(
                           page_number=2,
                           page_size=5,
                           order_by='-created_at',
                           filters={
                               'or': [
                                   {"created_at": {">": start_of_day}},
                                   {"created_at": {"<": end_of_day}}
                               ]
                           },
                           sentiment=None,  # sentiment 为 NULL
                           created_at={">": day}  # created_at > day
                       )
                       ```
                   """

        # Calculate the total number of records
        total_records = await self.get_count(filters=filters, **kwargs)

        # Apply limit, offset, and ordering for pagination
        query = select(self.model).limit(page_size).offset((page_number - 1) * page_size)

        # Apply filters and ordering
        query = self._apply_filters_and_ordering(query, order_by=order_by, filters=filters, **kwargs)

        res = await self.session.exec(query)
        elements = list(res.all())

        return Page(elements=elements, page_number=page_number, page_size=page_size, total_records=total_records)

    async def get_count(self, filters: dict = None, **kwargs: Any) -> int:
        """Gets the count of records with optional filters."""
        query = select(func.count()).select_from(self.model)

        # Apply filters using the same logic as _apply_filters
        query = self._apply_filters(query, filters=filters, **kwargs)

        res = await self.session.exec(query)
        return res.one()

    async def add(self, record: T) -> T:
        """Adds a new record to the database."""
        set_timestamps(record, is_new=True)
        self.session.add(record)
        await self.session.commit()
        await self.session.refresh(record)
        return record

    async def add_entity_if_not_exists(
            self,
            entity: T,
            unique_keys: dict[str, Any]
    ) -> T:
        """
        通用方法：添加实体（如果不存在）。
        检查实体是否存在，如果已存在则返回；否则添加新实体。
        """
        existing_entity = await self.get(**unique_keys)
        if existing_entity:
            return existing_entity
        return await self.add(entity)

    async def add_or_update_entity(
            self,
            entity: T,
            unique_keys: dict[str, Any],
            update_fields: list[str]
    ) -> T:
        """
        通用方法：添加或更新实体。
        如果实体存在，根据指定字段更新；否则添加新实体。
        """
        existing_entity = await self.get(**unique_keys)
        if existing_entity:
            for field in update_fields:
                setattr(existing_entity, field, getattr(entity, field))
            await self.update(existing_entity)
            return existing_entity
        return await self.add(entity)

    async def add_batch(self, records: List[T]) -> None:
        """
        批量添加记录到数据库。

        Args:
            records (List[T]): 要添加的记录列表。
        """
        if not records:
            return

        for record in records:
            set_timestamps(record, is_new=True)

        self.session.add_all(records)
        await self.session.commit()

    async def update(self, record: T) -> T:
        set_timestamps(record)
        """Updates an existing record in the     database."""
        self.session.add(record)
        await self.session.commit()
        await self.session.refresh(record)
        return record

    async def delete(self, *args: Any, **kwargs: Any) -> None:
        """Deletes a record from the database using its key."""
        record = await self.get(*args, **kwargs)
        if record:
            await self.session.delete(record)
            await self.session.commit()

    def _build_query(self, *args: Any, **kwargs: Any):
        """Helper function to build query based on primary keys or filters."""
        if args:
            primary_keys = get_primary_keys(self.model)
            if len(args) != len(primary_keys):
                raise ValueError(f"Expected {len(primary_keys)} primary key values, got {len(args)}.")
            pk_filter = dict(zip(primary_keys, args))
            return select(self.model).filter_by(**pk_filter)
        elif kwargs:
            return select(self.model).filter_by(**kwargs)
        else:
            raise ValueError(
                "Either primary key arguments (*args) or filtering conditions (**kwargs) must be provided.")

    def _apply_filters_and_ordering(self, query, filters: dict = None, order_by: str | List[str] | None = None,
                                    **kwargs: Any):
        """Apply dynamic filters and ordering to a SQLAlchemy query."""

        # Apply both complex filters (filters dict) and simple filters (kwargs)
        query = self._apply_filters(query, filters=filters, **kwargs)

        # Apply ordering
        if order_by:
            if isinstance(order_by, str):
                order_by = [order_by]  # Ensure order_by is a list for consistent processing

            order_criteria = []
            for order in order_by:
                if order.startswith('-'):
                    field_name = order[1:]  # Remove the '-' to get the field name
                    try:
                        order_criteria.append(desc(getattr(self.model, field_name)))
                    except AttributeError:
                        raise ValueError(f"Invalid field for ordering: {field_name}")
                else:
                    try:
                        order_criteria.append(asc(getattr(self.model, order)))
                    except AttributeError:
                        raise ValueError(f"Invalid field for ordering: {order}")

            query = query.order_by(*order_criteria)

        return query

    def _apply_filters(self, query, filters: dict = None, **kwargs: Any):
        """Apply both complex filters (AND/OR) and simple keyword-based filters to a SQL query."""

        # 处理 filters 和 kwargs
        filter_conditions = self._build_filter_conditions(filters) if filters else None

        # 检查简单条件 (kwargs)
        simple_conditions = []
        for key, value in kwargs.items():
            if value is None:
                simple_conditions.append(getattr(self.model, key).is_(None))
            elif isinstance(value, dict):
                for operator, operand in value.items():
                    condition = apply_operator(getattr(self.model, key), operator, operand)
                    if condition is not None:
                        simple_conditions.append(condition)
            else:
                # 默认处理为等于条件
                simple_conditions.append(getattr(self.model, key) == value)

        if filter_conditions is not None:
            query = query.filter(filter_conditions)

        if simple_conditions:
            query = query.filter(and_(*simple_conditions))

        return query

    def _build_filter_conditions(self, filters: dict) -> Any:
        """Build complex filter conditions from nested dictionaries (AND/OR logic)."""

        def parse_conditions(conditions, logic_type):
            """Recursively parse conditions with specified logic (AND/OR)."""
            parsed_conditions = []
            for condition in conditions:
                if isinstance(condition, dict):
                    parsed_conditions.append(self._build_filter_conditions(condition))
                else:
                    parsed_conditions.append(condition)
            # Filter out None conditions and apply the logic
            parsed_conditions = [cond for cond in parsed_conditions if cond is not None]
            return and_(*parsed_conditions) if logic_type == 'and' else or_(*parsed_conditions)

        # Handle 'and'/'or' at the top level
        if 'and' in filters:
            return parse_conditions(filters['and'], 'and')
        elif 'or' in filters:
            return parse_conditions(filters['or'], 'or')
        else:
            # Base case: single conditions
            return and_(*self._parse_single_conditions(filters))

    def _parse_single_conditions(self, conditions: dict) -> list:
        """Parse a single-level condition dict like {"age": {">": 18}}."""
        parsed_conditions = []
        for field, value in conditions.items():
            if isinstance(value, dict):
                for operator, operand in value.items():
                    condition = apply_operator(getattr(self.model, field), operator, operand)
                    parsed_conditions.append(condition)
            else:
                parsed_conditions.append(getattr(self.model, field) == value)
        return parsed_conditions


class MongoDBRepository(GenericRepository[T], ABC):
    """Base repository for MongoDB with async CRUD operations."""

    ID_FIELD = "id"
    MONGO_ID_FIELD = "_id"

    def __init__(self, db: AsyncIOMotorDatabase, model: Type[T]):
        self.model = model
        self.collection: AsyncIOMotorCollection = db[model.get_collection_name()]

    async def get(self, *args: Any, **kwargs: Any) -> T | None:
        """Retrieve a single record based on query."""

        query = self._build_query(*args, **kwargs)
        document = await self.collection.find_one(query)
        return self._deserialize(document)

    async def get_list(self, limit: int = 0, order_by: str | List[str] | None = None, **filters) -> List[T]:
        """Retrieve a list of records."""

        query = self._build_query(**filters)
        cursor = self.collection.find(query)

        if order_by:
            cursor = cursor.sort(mongo_build_sort(order_by))

        if limit > 0:
            cursor = cursor.limit(limit)

        documents = await cursor.to_list()
        return [self._deserialize(doc) for doc in documents]

    async def get_page_list(
        self, page_number: int = 1, page_size: int = 10, order_by: str | List[str] | None = None, **filters: Any
    ) -> Page[T]:
        """Retrieve a paginated list of records."""

        query = self._build_query(**filters)
        cursor = self.collection.find(query)

        if order_by:
            cursor = cursor.sort(mongo_build_sort(order_by))

        total_records = await self.collection.count_documents(query)
        cursor = cursor.skip((page_number - 1) * page_size).limit(page_size)
        documents = await cursor.to_list(length=page_size)

        return Page(elements=[self._deserialize(doc) for doc in documents],
                    page_number=page_number,
                    page_size=page_size,
                    total_records=total_records)

    async def get_count(self, **filters: Any) -> int:
        """Count the number of records matching the filters."""

        query = self._build_query(**filters)
        return await self.collection.count_documents(query)

    async def add(self, record: T) -> T:
        """Insert a new record."""

        document = record.model_dump(by_alias=True)
        document[self.MONGO_ID_FIELD] = document.pop(self.ID_FIELD)
        await self.collection.insert_one(document)
        return self._deserialize(document)

    async def update(self, record: T) -> T:
        """Update an existing record."""

        query = {self.MONGO_ID_FIELD: record.id}
        update_data = {"$set": record.model_dump(by_alias=True, exclude={self.ID_FIELD})}
        result = await self.collection.update_one(query, update_data)
        if result.matched_count == 0:
            raise ValueError("No document matched the given ID.")
        return record

    async def delete(self, *args: Any, **kwargs: Any) -> None:
        """Delete a record."""

        query = self._build_query(*args, **kwargs)
        result = await self.collection.delete_one(query)
        if result.deleted_count == 0:
            raise ValueError("No document matched the given criteria.")

    def _build_query(self, *args: Any, **kwargs: Any) -> dict:
        """
        Build MongoDB query from positional and keyword arguments.
        """

        query = {}

        # Handle positional arguments
        field_order = self.model.get_field_order()
        if args:
            if len(args) > len(field_order):
                raise ValueError(f"Too many positional arguments. Expected at most {len(field_order)}, got {len(args)}.")

            for field, value in zip(field_order, args):
                if field == self.ID_FIELD:
                    try:
                        query[self.MONGO_ID_FIELD] = ObjectId(value)
                    except:
                        query[self.MONGO_ID_FIELD] = value
                else:
                    query[field] = value

        # Handle keyword arguments
        if self.ID_FIELD in kwargs:
            try:
                query[self.MONGO_ID_FIELD] = ObjectId(kwargs.pop(self.ID_FIELD))
            except:
                query[self.MONGO_ID_FIELD] = kwargs.pop(self.ID_FIELD)

        query.update(kwargs)
        return query

    def _deserialize(self, document: dict) -> T | None:
        if document:
            document[self.ID_FIELD] = document.pop(self.MONGO_ID_FIELD)
            return self.model(**document)
        return None



from bson import ObjectId
from datetime import datetime, timezone
from typing import Optional, TypeVar, Generic

from pydantic import Field, BaseModel

T = TypeVar("T", bound=BaseModel)


class TimestampMixin:
    """创建时间&更新时间定义"""
    created_at: datetime = Field(default_factory=lambda: datetime.now(timezone.utc))
    updated_at: Optional[datetime] = None


class Page(BaseModel, Generic[T]):
    """Pagination result data structure."""

    elements: list[T]  # Data
    page_number: int  # Page number
    page_size: int  # Number of records per page
    total_records: int  # Total number of records


class BaseMongoModel(BaseModel):
    """基础 MongoModel"""

    id: Optional[ObjectId] = Field(default_factory=ObjectId)

    @classmethod
    def get_field_order(cls) -> list[str]:
        return ["id"]

    @classmethod
    def get_collection_name(cls) -> str:
        """Override this method in derived models to provide collection name."""
        raise NotImplementedError("Collection name is not defined for this model.")

    class Config:
        populate_by_name = True
        arbitrary_types_allowed = True
        json_encoders = {ObjectId: str}





import random
import string
from datetime import datetime, timezone


def rand_str(n: int = 8, options: str = string.ascii_letters + string.digits) -> str:
    return ''.join(random.sample(options, n))


def generate_hash_code() -> str:
    now_time = int(datetime.now(timezone.utc).timestamp() * 1000 * 1000)
    pre_str = format(now_time, "x")
    pre_str = pre_str.zfill(16)
    return pre_str + rand_str(16)




from vovo.cache.cache_manager import Cache
from vovo.cache.custom_key_maker import CustomKeyMaker
from vovo.cache.redis_backend import RedisBackend

__all__ = [
    "Cache",
    "RedisBackend",
    "CustomKeyMaker",
]


import httpx

DEFAULT_CONNECTION_TIMEOUT: int = 30
DEFAULT_REQUEST_TIMEOUT: int = 100


class VovoClientFactory:
    """Http client factory"""

    @staticmethod
    def get_client(base_url: str = '', headers: dict = None) -> httpx.AsyncClient:
        """Returns a native HTTP AsyncClient(httpx.AsyncClient) instance

          Args:
            base_url (str): The base URL for the client.
            headers (dict, optional): Custom headers to include in the requests. Defaults to None.

        Returns:
            httpx.AsyncClient
        """
        timeout = httpx.Timeout(DEFAULT_REQUEST_TIMEOUT, connect=DEFAULT_CONNECTION_TIMEOUT)
        return httpx.AsyncClient(timeout=timeout, http2=True, base_url=base_url, headers=headers)



from fastapi import Request
from fastapi.responses import JSONResponse

from vovo.core.api import APIResponse, APIError
from vovo.exceptions import VovoBusinessException


async def business_exception_handler(request: Request, exc: Exception):
    """业务异常处理"""

    if isinstance(exc, VovoBusinessException):
        return JSONResponse(
            status_code=exc.status_code,
            content=APIResponse(success=False, error=APIError(code=exc.code, message=exc.detail),
                                data=None).model_dump())
    else:
        return JSONResponse(
            status_code=500,
            content={"message": "An unexpected error occurred"}
        )


import os
from dataclasses import is_dataclass, asdict

import pandas as pd
import requests
from pydantic import BaseModel
from tqdm import tqdm


def download_file_with_temporary_save(url, save_path):
    """下载文件并保存"""

    temp_save_path = save_path + ".part"  # 使用临时文件路径

    try:
        # 发送HTTP请求获取文件
        response = requests.get(url, stream=True)
        response.raise_for_status()  # 如果请求不成功，会抛出异常

        # 获取文件总大小
        total_size = int(response.headers.get('content-length', 0))

        # 打开文件以二进制写入
        with open(temp_save_path, 'wb') as file:
            # 使用 tqdm 显示进度条
            with tqdm(total=total_size, unit='B', unit_scale=True, desc=temp_save_path, initial=0, ascii=True) as pbar:
                for chunk in response.iter_content(chunk_size=8192):
                    if chunk:  # 过滤掉保持活动的空块
                        file.write(chunk)
                        pbar.update(len(chunk))

        # 下载完成后重命名临时文件为最终文件
        os.rename(temp_save_path, save_path)
    except Exception:
        if os.path.exists(temp_save_path):
            os.remove(temp_save_path)  # 删除临时文件
        raise


def save_to_csv(data, filename='output.csv'):
    """定义一个函数来转换 dataclass | pydantic model 数据到 DataFrame 并保存为 CSV"""
    # 检查每个记录的类型并进行适当的转换
    records_dict = []
    for record in data:
        if isinstance(record, BaseModel):
            # 如果是 Pydantic 模型
            records_dict.append(record.model_dump())
        elif is_dataclass(record):
            # 如果是标准 dataclass
            records_dict.append(asdict(record))
        else:
            raise TypeError("Unsupported data type")

    # 将字典列表转换为 DataFrame,保存为 CSV 文件
    pd.DataFrame(records_dict).to_csv(filename, index=False)


def get_file_extension(filename):
    """
    获取文件的扩展名。

    参数:
        filename (str): 文件名或文件路径。

    返回:
        str: 文件的扩展名（包括点），如果没有扩展名，则返回空字符串。
    """
    _, extension = os.path.splitext(filename)
    return extension




from typing import Any, Union


class VovoException(Exception):
    def __init__(self, detail: str, code: Union[int, str] = 400):
        self.code = code
        self.details = detail


class VovoBusinessException(VovoException):
    """Vovo 业务异常的基类"""

    def __init__(self, detail: str, code: Union[int, str] = "BusinessError", status_code: int = 400):
        self.detail = detail
        self.code = code
        self.status_code = status_code


class VovoHttpException(VovoException):
    """Vovo HTTP 异常的基类"""


class ParserException(ValueError, VovoException):
    """解析器应引发异常以表示解析错误"""

    def __init__(self, error: Any):
        super(ParserException, self).__init__(error)



import traceback
import uuid
import logging
from typing import Optional, Generic, TypeVar, Union, ParamSpec, Callable
from functools import update_wrapper


from pydantic import BaseModel

ResponseType = TypeVar("ResponseType")

R = TypeVar("R")
T = TypeVar("T")
P = ParamSpec("P")
F = TypeVar("F")


class APIError(BaseModel):
    """定义错误信息结构"""
    code: Union[int, str]
    message: str


class APIResponse(BaseModel, Generic[ResponseType]):
    """API响应结构"""
    data: Optional[ResponseType]
    error: Optional[APIError] = None
    traceId: Optional[str] = None
    success: bool = True


def with_exception_handling(f: Callable[P, R]) -> Callable[P, APIResponse[R]]:
    """Marks a callback as wanting to receive the current context object as first argument."""

    def wrapper(*args: P.args, **kwargs: P.kwargs) -> APIResponse[R]:
        trace_id = str(uuid.uuid4())
        try:
            return APIResponse[R](data=f(*args, **kwargs))
        except Exception as e:
            logging.getLogger().error(f"TraceId: {trace_id}\n{traceback.format_exc()}")
            return APIResponse[R](data=None, error=APIError(code=500, message=str(e) or "Internal Server Error"),
                                  traceId=trace_id)

    return update_wrapper(wrapper, f)



from abc import ABC, abstractmethod
from typing import Any


class BaseBackend(ABC):
    @abstractmethod
    async def get(self, *, key: str) -> Any:
        """Get"""

    @abstractmethod
    async def set(self, *, response: Any, key: str, ttl: int = 60) -> None:
        """Set"""

    @abstractmethod
    async def delete_startswith(self, *, value: str) -> None:
        """Delete starts with"""



from abc import ABC, abstractmethod
from typing import Any, Callable, Dict, Generic, Iterator, List, Optional, TypeVar, Union, TypedDict

from vovo.custom_logger import logger

Input = TypeVar("Input")
Output = TypeVar("Output")


class Runnable(Generic[Input, Output], ABC):
    """一个基础可运行单元（Runnable），支持管道式操作、批处理和重试等特性。"""

    def __or__(self, other: Union["Runnable", Callable]) -> "RunnableSequence":
        """支持管道操作符 `|`，并自动将普通函数转换为 `LambdaRunnable`。"""
        if not isinstance(other, Runnable):
            other = LambdaRunnable(other)  # 自动封装为 Runnable
        return RunnableSequence([self, other])

    def map(self, inputs: List[Input]) -> List[Output]:
        """批量处理多个输入，返回对应的多个输出。"""
        return [self.invoke(i) for i in inputs]

    def with_retry(self, attempts: int = 3) -> "RetryRunnable":
        """为当前 Runnable 添加重试机制。"""
        return RetryRunnable(self, attempts)

    @abstractmethod
    def invoke(self, input: Input) -> Output:
        """实现单次调用的逻辑。"""
        pass


class RunnableSequence(Runnable):
    """将多个 Runnable 顺序组合为一个运行链。"""

    def __init__(self, runnable_list: List[Runnable]):
        self.runnable_list = runnable_list

    def invoke(self, input: Any) -> Any:
        """依次执行每个 Runnable，将输出作为下一个的输入。"""
        output = input
        for runnable in self.runnable_list:
            output = runnable.invoke(output)
        return output


class RetryRunnable(Runnable):
    """包装一个 Runnable，为其添加重试机制。"""

    def __init__(self, runnable: Runnable, attempts: int):
        self.runnable = runnable
        self.attempts = attempts

    def invoke(self, input: Input) -> Output:
        """尝试多次执行 Runnable，直到成功或达到最大重试次数。"""
        last_exception = None
        for attempt in range(self.attempts):
            try:
                return self.runnable.invoke(input)
            except Exception as e:
                last_exception = e
                logger.error(f"Retry {attempt + 1} failed: {e}")
        raise last_exception


class LambdaRunnable(Runnable):
    """一个简单的 Runnable，用于包装任意的函数。"""

    def __init__(self, func: Callable[[Input], Output]):
        self.func = func

    def invoke(self, input: Input) -> Output:
        return self.func(input)


class RouterInput(TypedDict):
    """Router input.

    Attributes:
        key: The key to route on.
        input: The input to pass to the selected Runnable.
    """
    key: str
    input: Any


class RouterRunnable(Runnable[RouterInput, Output]):
    """
    一个基于 `key` 的路由 Runnable。
    根据 `RouterInput['key']` 选择对应的 Runnable 并执行。
    """

    def __init__(self, runnables: Dict[str, Runnable[Any, Any]]):
        self.runnables = runnables

    def invoke(self, input: RouterInput) -> Output:
        """路由到对应的 Runnable 并执行。"""
        if "key" not in input or "input" not in input:
            raise ValueError("Input must contain 'key' and 'input' fields.")

        key = input["key"]
        inner_input = input["input"]

        if key not in self.runnables:
            raise KeyError(f"Key '{key}' not found in runnables: {list(self.runnables.keys())}")

        runnable = self.runnables[key]

        output = runnable.invoke(inner_input)

        return output



import logging
import uuid

import structlog
import sys
from colorama import init

# 初始化 colorama 以支持 Windows 系统上的颜色显示
init(autoreset=True)

logger = structlog.get_logger()


def configure_logging(log_format="key_value", log_level=logging.INFO, use_colors=True):
    """
    配置 structlog 日志记录器。

    :param use_colors: 是否启用颜色输出（默认启用）
    :param log_format: 日志输出格式 ('key_value' 或 'json')
    :param log_level: 日志级别，默认是 logging.INFO
    """

    # 配置 Python 原生 logging
    logging.basicConfig(
        format="%(message)s",
        stream=sys.stdout,
        level=log_level,
    )

    # 选择输出格式（KeyValueRenderer 或 JSONRenderer）
    if log_format == "json":
        renderer = structlog.processors.JSONRenderer()
    else:
        renderer = structlog.dev.ConsoleRenderer(colors=use_colors)

    # 配置 structlog
    structlog.configure(
        processors=[
            structlog.stdlib.filter_by_level,  # 根据日志级别过滤
            structlog.stdlib.add_logger_name,  # 添加 logger 名称
            structlog.stdlib.add_log_level,  # 添加日志级别
            structlog.processors.StackInfoRenderer(),  # 如果有异常，添加栈信息
            structlog.processors.format_exc_info,  # 格式化异常信息
            structlog.processors.TimeStamper(fmt="iso"),  # 添加时间戳
            renderer,  # 使用选择的渲染器（支持颜色和多种格式）
        ],
        context_class=dict,
        logger_factory=structlog.stdlib.LoggerFactory(),
        cache_logger_on_first_use=True,
    )


def generate_correlation_id() -> str:
    return str(uuid.uuid4())
